package com.redxun.knowledge.analysis.service;

import com.redxun.knowledge.analysis.entity.consts.PlatformConst;
import com.redxun.knowledge.analysis.entity.vo.LablesVo;
import com.redxun.knowledge.analysis.entity.vo.SearchParticipleWordCloudVo;
import com.redxun.knowledge.analysis.entity.vo.SearchVisitHistogramVo;
import com.redxun.knowledge.analysis.mapper.AnalysisCommonMapper;
import com.redxun.knowledge.analysis.mapper.PvLogMapper;
import com.redxun.knowledge.analysis.mapper.SearchParticipleLogMapper;
import com.redxun.knowledge.utils.DateUtils;
import groovy.util.logging.Slf4j;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 文件名: CommonServiceImpl
 * 作者: zizg
 * 时间: 2023/3/17
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Service
@Slf4j
public class AnalysisCommonServiceImpl {

    @Autowired
    private AnalysisCommonMapper analysisCommonMapper;

    @Autowired
    private PvLogMapper pvLogMapper;

    /**
     * 各模块整体数量统计：知识仓库（知识总数量）；知识地图（地图总数量）；知识专辑（专辑总数量）
     * @return
     */
    public Map<String, Object> totalAmount() {
        Map<String, Object> result = new HashMap<>();
        Integer knowledgeTotal = analysisCommonMapper.knowledgeTotalAmount();
        Integer mapTotal = analysisCommonMapper.mapTotalAmount();
        Integer albumTotal = analysisCommonMapper.albumTotalAmount();
        result.put("knowledge",knowledgeTotal);
        result.put("map",mapTotal);
        result.put("album",albumTotal);
        return result;
    }

    /**
     * 查询各类型（维基、文档）知识数量
     * @return
     */
    public Map<String, Object> knowledgeTypeAmount() {
        Map<String, Object> result = new HashMap<>();
        //类型为1(文档知识)
        Integer archiveTotal = analysisCommonMapper.knowledgeTotalByType(1);
        //类型为2(维基知识)
        Integer wikiTotal = analysisCommonMapper.knowledgeTotalByType(2);
        result.put("archive",archiveTotal);
        result.put("wiki",wikiTotal);
        result.put("total",archiveTotal + wikiTotal);
        return result;
    }

    /**
     * 获取搜索服务分词的词云数据
     * @param tops
     * @return
     */
    public List<SearchParticipleWordCloudVo> searchParticipleWordCloud(Integer tops) {
        List<SearchParticipleWordCloudVo> searchParticipleWordCloudVos = analysisCommonMapper.searchParticipleWordCloud(tops);
        List<SearchParticipleWordCloudVo> result = searchParticipleWordCloudVos.stream().limit(tops).collect(Collectors.toList());
        return result;
    }

    /**
     * 获取搜索服务访问量分布柱状图数据，查看一次详情算一次PV
     * @return
     */
    public SearchVisitHistogramVo searchVisitHistogram() {
        SearchVisitHistogramVo searchVisitHistogramVo = new SearchVisitHistogramVo();
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        //统计当前年的搜索访问总量
        int yearTotal = pvLogMapper.selectYearTotal(DateUtils.format(DateUtils.getFirstOfYear(year)),
                DateUtils.format(DateUtils.getLastOfYear(year)));
        //统计每月各个系统的搜索访问量
        List<LablesVo> lablesVoList = new ArrayList<>();
        for (int i = 1; i <= 12 ; i++) {
            LablesVo lablesVo = new LablesVo();
            List<Map<String, Object>> values = new ArrayList<>();
            lablesVo.setName(i + "月");
            //查询出全部的业务系统名称
            List<String> platformList = PlatformConst.platformList();
            //获取每月的第一天和最后一天
            String firstOfMonth = DateUtils.getFirstOfMonth(year, i, 15);
            String lastOfMonth = DateUtils.getLastOfMonth(year, i, 15);
            Integer sumByMonth = pvLogMapper.selectSearchVisitHistogramByMonth(firstOfMonth, lastOfMonth, null);
            //根据月份查询各个业务系统访问量
            platformList.forEach(e -> {
                Map<String, Object> map = new HashMap<>();
                Integer count = pvLogMapper.selectSearchVisitHistogramByMonth(firstOfMonth,lastOfMonth,e);
                map.put("legend",e);
                map.put("value",count);
                if (sumByMonth == 0){
                    map.put("percentage",0.0000);
                } else {
                    double sum = count / (sumByMonth.doubleValue());
                    BigDecimal bd = new BigDecimal(sum);
                    double f1 = bd.setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
                    map.put("percentage",f1);
                }
                values.add(map);
            });
            lablesVo.setValues(values);
            lablesVoList.add(lablesVo);
        }
        //返回值
        searchVisitHistogramVo.setTotal(yearTotal);
        searchVisitHistogramVo.setLables(lablesVoList);
        return searchVisitHistogramVo;
    }
}
